package demo.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.seasar.doma.DomaNullPointerException;
import org.seasar.doma.bean.BeanPropertyWrapper;
import org.seasar.doma.bean.BeanWrapper;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.PropertyDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;

public class S2BeanWrapper implements BeanWrapper {

    protected final Object bean;

    protected final Class<?> beanClass;

    protected final List<BeanPropertyWrapper> propertyWrappers;

    protected final Map<String, BeanPropertyWrapper> propertyWrapperMap;

    public S2BeanWrapper(Object bean) throws DomaNullPointerException {
        if (bean == null) {
            throw new DomaNullPointerException("The bean parameter is null.");
        }
        this.bean = bean;
        this.beanClass = bean.getClass();
        this.propertyWrapperMap = createPropertyWrapperMap(beanClass);
        this.propertyWrappers = Collections
                .unmodifiableList(new ArrayList<BeanPropertyWrapper>(
                        propertyWrapperMap.values()));
    }

    @Override
    public Class<?> getBeanClass() {
        return beanClass;
    }

    @Override
    public BeanPropertyWrapper getBeanPropertyWrapper(String name) {
        return propertyWrapperMap.get(name);
    }

    @Override
    public List<BeanPropertyWrapper> getBeanPropertyWrappers() {
        return propertyWrappers;
    }

    protected Map<String, BeanPropertyWrapper> createPropertyWrapperMap(
            Class<?> beanClass) {
        LinkedHashMap<String, BeanPropertyWrapper> result = new LinkedHashMap<String, BeanPropertyWrapper>();
        BeanDesc beanDesc = BeanDescFactory.getBeanDesc(beanClass);
        for (int i = 0; i < beanDesc.getPropertyDescSize(); i++) {
            PropertyDesc propertyDesc = beanDesc.getPropertyDesc(i);
            S2PropertyWrapper propertyWrapper = new S2PropertyWrapper(
                    propertyDesc);
            String name = propertyWrapper.getName();
            result.put(name, propertyWrapper);
        }
        return result;
    }

    protected class S2PropertyWrapper implements BeanPropertyWrapper {

        protected final PropertyDesc propertyDesc;

        public S2PropertyWrapper(PropertyDesc propertyDesc) {
            this.propertyDesc = propertyDesc;
        }

        @Override
        public String getName() {
            return propertyDesc.getPropertyName();
        }

        @Override
        public Class<?> getPropertyClass() {
            return propertyDesc.getPropertyType();
        }

        @Override
        public boolean isValueGettable() {
            return propertyDesc.isReadable();
        }

        @Override
        public Object getValue() {
            return propertyDesc.getValue(bean);
        }

        @Override
        public boolean isValueSettable() {
            return propertyDesc.isWritable();
        }

        @Override
        public void setValue(Object value) {
            propertyDesc.setValue(bean, value);
        }
    }

}
